package dev.fyloz.colorrecipesexplorer.dtos

import com.fasterxml.jackson.annotation.JsonIgnore
import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.SpringUserDetails
import dev.fyloz.colorrecipesexplorer.model.account.Permission
import dev.fyloz.colorrecipesexplorer.model.account.toAuthority
import java.time.LocalDateTime
import javax.validation.constraints.NotBlank
import javax.validation.constraints.Size

data class UserDto(
    override val id: Long = 0L,

    val firstName: String,

    val lastName: String,

    @field:JsonIgnore
    val password: String = "",

    val group: GroupDto?,

    val permissions: List<Permission>,

    val explicitPermissions: List<Permission> = listOf(),

    val lastLoginTime: LocalDateTime? = null,

    @field:JsonIgnore
    val isDefaultGroupUser: Boolean = false,

    @field:JsonIgnore
    val isSystemUser: Boolean = false
) : EntityDto {
    @get:JsonIgnore
    val authorities
        get() = permissions
            .map { it.toAuthority() }
            .toMutableSet()
}

data class UserSaveDto(
    val id: Long = 0L,

    @field:NotBlank
    val firstName: String,

    @field:NotBlank
    val lastName: String,

    @field:NotBlank
    @field:Size(min = 8, message = Constants.ValidationMessages.PASSWORD_TOO_SMALL)
    val password: String,

    val groupId: Long?,

    val permissions: List<Permission>,

    // TODO WN: Test if working
    // @JsonProperty(access = JsonProperty.Access.READ_ONLY)
    @field:JsonIgnore
    val isSystemUser: Boolean = false,

    @field:JsonIgnore
    val isDefaultGroupUser: Boolean = false
)

data class UserUpdateDto(
    val id: Long = 0L,

    @field:NotBlank
    val firstName: String,

    @field:NotBlank
    val lastName: String,

    val groupId: Long?,

    val permissions: List<Permission>
)

data class UserLoginRequestDto(val id: Long, val password: String)

class UserDetails(val user: UserDto) : SpringUserDetails {
    override fun getPassword() = user.password
    override fun getUsername() = user.id.toString()
    override fun getAuthorities() = user.authorities

    override fun isAccountNonExpired() = true
    override fun isAccountNonLocked() = true
    override fun isCredentialsNonExpired() = true
    override fun isEnabled() = true
}