package dev.fyloz.colorrecipesexplorer.service

import dev.fyloz.colorrecipesexplorer.config.annotations.ServiceComponent
import dev.fyloz.colorrecipesexplorer.dtos.MixMaterialDto
import dev.fyloz.colorrecipesexplorer.model.MixMaterial
import dev.fyloz.colorrecipesexplorer.repository.MixMaterialRepository

interface MixMaterialService : Service<MixMaterialDto, MixMaterial, MixMaterialRepository> {
    /** Checks if a mix material with the given [materialId] exists. */
    fun existsByMaterialId(materialId: Long): Boolean
}

@ServiceComponent
class DefaultMixMaterialService(repository: MixMaterialRepository, private val materialService: MaterialService) :
    BaseService<MixMaterialDto, MixMaterial, MixMaterialRepository>(repository), MixMaterialService {
    override fun existsByMaterialId(materialId: Long) = repository.existsByMaterialId(materialId)

    override fun toDto(entity: MixMaterial) =
        MixMaterialDto(entity.id, materialService.toDto(entity.material), entity.quantity, entity.position)

    override fun toEntity(dto: MixMaterialDto) =
        MixMaterial(dto.id, materialService.toEntity(dto.material), dto.quantity, dto.position)
}